/****************************************************************************
** $Id: qt/examples/application/application.cpp   2.2.4   edited 2000-08-31 $
**
** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
**
** This file is part of an example program for Qt.  This example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/

#include "application.h"

// language file
#include "language.h"
// end language include(s)

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <qimage.h>
#include <qbitmap.h>
#include <qpixmap.h>
#include <qtooltip.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qmultilineedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qaccel.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qpushbutton.h>
#include <qcolordialog.h>
#include <qpaintdevicemetrics.h>
#include <qwhatsthis.h>
#include <qscrollview.h>
#include <qcombobox.h>
#include <qfont.h>
#include <qdir.h>
#include <qstringlist.h>
#include <qmime.h>

#include "defs.h"
#include "drawable.h"
#include "render2d.h"
#include "chemdata.h"
#include "ringdialog.h"
#include "fixeddialog.h"
#include "netdialog.h"
#include "netchoosedialog.h"
#include "netaccess.h"
#include "pagesetupdialog.h"
#include "xdc_toolbutton.h"

// top toolbar
#include "filesave.xpm"
#include "fileopen.xpm"
#include "fileprint.xpm"
#include "cuttool.xpm"
#include "copytool.xpm"
#include "pastetool.xpm"

// line thicknesses
#include "lines.h"

// text operation buttons
#include "boldtool.xpm"
#include "italictool.xpm"
#include "underlinetool.xpm"
#include "superscript.xpm"
#include "subscript.xpm"

// left side toolbar
#include "selecttool.xpm"
#include "erasetool.xpm"
#include "linetool.xpm"
#include "dashtool.xpm"
#include "uptool.xpm"
#include "downtool.xpm"
#include "texttool.xpm"
#include "arrowtool.xpm"
#include "curvearrowtool.xpm"
#include "brackettool.xpm"
#include "ringtool.xpm"
#include "symboltool.xpm"

// predefined toolbar
#include "rings.h"

// Brackets
#include "brackets.h"
// end brackets

// Symbols
#include "symbol_xpm.h"
// end symbols

// Arrows
#include "arrows.h"
#include "cw90.xpm"
#include "ccw90.xpm"
#include "cw180.xpm"
#include "ccw180.xpm"
#include "cw270.xpm"
#include "ccw270.xpm"
// end Arrows

// defined in main.cpp
extern QString RingDir;

// note that all What's This? text is now in the language_*.h file

// What's This? text for top toolbar
const char * fileOpenText = FILE_OPEN_TEXT;
const char * fileSaveText = FILE_SAVE_TEXT;
const char * filePrintText = FILE_PRINT_TEXT;
const char * editCutText = EDIT_CUT_TEXT;
const char * editCopyText = EDIT_COPY_TEXT;
const char * editPasteText = EDIT_PASTE_TEXT;

// What's This? text for left side toolbar
const char * selectToolText = SELECT_TOOL_TEXT;
const char * eraseToolText = ERASE_TOOL_TEXT;
const char * lineToolText = LINE_TOOL_TEXT;
const char * dashLineToolText = DASH_LINE_TOOL_TEXT;
const char * upLineToolText = UP_LINE_TOOL_TEXT;
const char * downLineToolText = DOWN_LINE_TOOL_TEXT;
const char * arrowToolText = ARROW_TOOL_TEXT;
const char * cArrowToolText = CURVE_ARROW_TOOL_TEXT;
const char * bracketToolText = BRACKET_TOOL_TEXT;
const char * textToolText = TEXT_TOOL_TEXT;
const char * ringToolText = RING_TOOL_TEXT;
const char * symbolToolText = SYMBOL_TOOL_TEXT;

ApplicationWindow::ApplicationWindow(Clipboard *clipin)
    : QMainWindow( 0, "example application main window", WDestructiveClose )
{
  clip = clipin;
  int id;
  QMimeSourceFactory *myfactory = QMimeSourceFactory::defaultFactory();

  printer = new QPrinter;
  QPixmap openIcon, saveIcon, printIcon, superIcon, subIcon;
  QPixmap swatchblack, swatchred, swatchgreen, swatchblue;
  QPixmap selectIcon, drawBoxIcon, drawLineIcon, downLineIcon, upLineIcon,
    textIcon, arrowIcon, bracketIcon, cutIcon, copyIcon, pasteIcon, ringIcon,
    dashIcon, symbolIcon, curveArrowIcon;

  // create scrolling viewport and render widget
  sv = new QScrollView(this);
  r = new Render2D( sv->viewport(), "render2d" );
  sv->addChild(r);

  // top toolbar
  fileTools = new QToolBar( this, "file operations" );
  fileTools->setLabel( tr( "File Operations" ) );

  openIcon = QPixmap( fileopen );
  myfactory->setPixmap("fileopenicon", openIcon );
  QToolButton * fileOpen
    = new QToolButton( openIcon, TOOLTIP_OPEN_FILE, QString::null,
		       this, SLOT(load()), fileTools, "open file" );
  QWhatsThis::add(fileOpen, fileOpenText);

  saveIcon = QPixmap( filesave );
  myfactory->setPixmap("filesaveicon", saveIcon );
  QToolButton * fileSave
    = new QToolButton( saveIcon, TOOLTIP_SAVE_FILE, QString::null,
		       this, SLOT(save()), fileTools, "save file" );
  QWhatsThis::add(fileSave, fileSaveText);

  printIcon = QPixmap( fileprint );
  myfactory->setPixmap("fileprinticon", printIcon );
  QToolButton * filePrint
    = new QToolButton( printIcon, TOOLTIP_PRINT_FILE, QString::null,
		       this, SLOT(print()), fileTools, "print file" );
  QWhatsThis::add(filePrint, filePrintText);

  cutIcon = QPixmap(cuttool);
  myfactory->setPixmap("cuticon", cutIcon );
  QToolButton * editCut
    = new QToolButton( cutIcon, TOOLTIP_CUT, QString::null,
		       this, SLOT(Cut()), fileTools, "cut" );
  QWhatsThis::add(editCut, editCutText);

  copyIcon = QPixmap(copytool);
  myfactory->setPixmap("copyicon", copyIcon );
  QToolButton * editCopy
    = new QToolButton( copyIcon, TOOLTIP_COPY, QString::null,
		       this, SLOT(Copy()), fileTools, "copy" );
  QWhatsThis::add(editCopy, editCopyText);

  pasteIcon = QPixmap(pastetool);
  myfactory->setPixmap("pasteicon", pasteIcon );
  QToolButton * editPaste
    = new QToolButton( pasteIcon, TOOLTIP_PASTE, QString::null,
		       this, SLOT(Paste()), fileTools, "paste" );
  QWhatsThis::add(editPaste, editPasteText);

  QPixmap *tmp_pm = new QPixmap(32, 20);
  tmp_pm->fill(QColor(210,210,210));
  QPainter pixpaint(tmp_pm);
  pixpaint.fillRect(2, 3, 14, 14, QColor(0,0,0));
  pixpaint.setPen(QColor(0,0,0));
  pixpaint.drawLine( 24, 2, 24, 16 );
  pixpaint.drawLine( 24, 16, 20, 12 );
  pixpaint.drawLine( 24, 16, 28, 12 );
  pb1 = new QPushButton(fileTools);
  pb1->setPixmap(*tmp_pm);
  QToolTip::add(pb1, TOOLTIP_SET_COLOR);
  connect(pb1, SIGNAL(pressed()), this, SLOT(NewColor()) );

  ltList = new QComboBox(false, fileTools);
  QToolTip::add(ltList, TOOLTIP_SET_THICKNESS);
  QWhatsThis::add(ltList, tr("Set Line Thickness") );
  QPixmap px1;
  QBitmap mask0(12, 12);
  mask0.fill(Qt::color0);
  px1 = QPixmap(line1);
  px1.setMask(mask0);
  ltList->insertItem(line1, "1");
  ltList->insertItem(line2, "2");
  ltList->insertItem(line3, "3");
  ltList->insertItem(line4, "4");
  ltList->insertItem(line5, "5");
  connect(ltList, SIGNAL(activated(int)), this, SLOT(SetThick(int)) );

  fontList = new QComboBox(false, fileTools);
  QToolTip::add(fontList, TOOLTIP_SET_FONT);
  QWhatsThis::add(fontList, tr("Set Font") );
  fontList->insertItem( "Courier" );
  fontList->insertItem( "Helvetica" );
  fontList->insertItem( "Times" );
  fontList->setCurrentItem(1);
  connect(fontList, SIGNAL(activated(int)), this, SLOT(SetFont(int)) );

  fontSizeList = new QComboBox(false, fileTools);
  QToolTip::add(fontSizeList, TOOLTIP_SET_FONT_SIZE);
  QWhatsThis::add(fontSizeList, tr("Set Font Size") );
  fontSizeList->insertItem(QString("8"));
  fontSizeList->insertItem(QString("10"));
  fontSizeList->insertItem(QString("12"));
  fontSizeList->insertItem(QString("14"));
  fontSizeList->insertItem(QString("18"));
  fontSizeList->insertItem(QString("24"));
  fontSizeList->insertItem(QString("32"));
  fontSizeList->setCurrentItem(2);
  connect(fontSizeList, SIGNAL(activated(int)), this, SLOT(SetFont(int)) );

  superIcon = QPixmap( boldtool_xpm );
  myfactory->setPixmap("boldtool", superIcon );
  Bold
    = new QToolButton( superIcon, TOOLTIP_BOLD, QString::null,
		       r, SLOT(Bold()), fileTools, "bold" );
  Bold->hide();

  superIcon = QPixmap( italictool_xpm );
  myfactory->setPixmap("italictool", superIcon );
  Italic
    = new QToolButton( superIcon, TOOLTIP_ITALIC, QString::null,
		       r, SLOT(Italic()), fileTools, "italic" );
  Italic->hide();

  superIcon = QPixmap( underlinetool_xpm );
  myfactory->setPixmap("underlinetool", superIcon );
  Underline
    = new QToolButton( superIcon, TOOLTIP_UNDERLINE, QString::null,
		       r, SLOT(Underline()), fileTools, "underline" );
  Underline->hide();

  superIcon = QPixmap( superscript_xpm );
  myfactory->setPixmap("supericon", superIcon );
  superScript
    = new QToolButton( superIcon, TOOLTIP_SUPERSCRIPT, QString::null,
		       r, SLOT(Superscript()), fileTools, "superscript" );
  superScript->hide();

  subIcon = QPixmap( subscript_xpm );
  myfactory->setPixmap("subicon", subIcon );
  subScript
    = new QToolButton( subIcon, TOOLTIP_SUBSCRIPT, QString::null,
		       r, SLOT(Subscript()), fileTools, "subscript" );
  subScript->hide();

  QToolButton *wt = QWhatsThis::whatsThisButton( fileTools );
  myfactory->setPixmap( "whatsthisicon", *(wt->pixmap()) );

  // left side toolbar
  drawTools = new QToolBar(this, "draw operations");
  moveToolBar(drawTools, QMainWindow::Left);
  selectIcon = QPixmap( selecttool );
  myfactory->setPixmap("selecttoolicon", selectIcon );
  QToolButton * selectObjects
    = new QToolButton( selectIcon, TOOLTIP_SELECT, QString::null,
		       r, SLOT(setMode_Select()), drawTools, "select" );
  QWhatsThis::add(selectObjects, selectToolText);

  drawBoxIcon = QPixmap( eraseIcon );
  myfactory->setPixmap("erasetoolicon", drawBoxIcon );
  QToolButton * drawBox
    = new QToolButton( drawBoxIcon, TOOLTIP_ERASE, QString::null,
		       r, SLOT(setMode_Erase()), drawTools, "erase" );
  QWhatsThis::add(drawBox, eraseToolText);

  drawLineIcon = QPixmap( linetool );
  myfactory->setPixmap("linetoolicon", drawLineIcon );
  QToolButton * drawLine
    = new QToolButton( drawLineIcon, TOOLTIP_LINE, QString::null,
		       r, SLOT(setMode_DrawLine()), drawTools, 
		       "draw line" );
  QWhatsThis::add(drawLine, lineToolText);

  dashIcon = QPixmap( dashtool );
  myfactory->setPixmap("dashtoolicon", dashIcon );
  QToolButton * drawDashLine
    = new QToolButton( dashIcon, TOOLTIP_DASH_LINE, QString::null,
		       r, SLOT(setMode_DrawDashLine()), drawTools, 
		       "draw dashed line" );
  QWhatsThis::add(drawDashLine, dashLineToolText);

  upLineIcon = QPixmap( uptool );
  myfactory->setPixmap("uplinetoolicon", upLineIcon );
  QToolButton * drawUpLine
    = new QToolButton( upLineIcon, TOOLTIP_UPLINE, QString::null,
		       r, SLOT(setMode_DrawUpLine()), drawTools, 
		       "draw stereo-up line" );
  QWhatsThis::add(drawUpLine, upLineToolText);

  downLineIcon = QPixmap( downtool );
  myfactory->setPixmap("downlinetoolicon", downLineIcon );
  QToolButton * drawDownLine
    = new QToolButton( downLineIcon, TOOLTIP_DOWNLINE, QString::null,
		       r, SLOT(setMode_DrawDownLine()), drawTools, 
		       "draw stereo-down line" );
  QWhatsThis::add(drawDownLine, downLineToolText);

  arrowIcon = QPixmap( arrowtool );
  myfactory->setPixmap("arrowtoolicon", arrowIcon );
  QToolButton * drawArrowLine
    = new QToolButton( arrowIcon, TOOLTIP_ARROW, QString::null,
		       this, SLOT(DrawRegularArrow()), drawTools, 
		       "draw arrow" );
  drawArrowLine->setPopup(BuildArrowMenu());
  drawArrowLine->setPopupDelay(150);
  QWhatsThis::add(drawArrowLine, arrowToolText);

  curveArrowIcon = QPixmap( curvearrowtool_xpm );
  myfactory->setPixmap("curvearrowtoolicon", curveArrowIcon );
  QToolButton * drawCArrow
    = new QToolButton( curveArrowIcon, TOOLTIP_CURVEARROW, QString::null,
		       this, SLOT(Dummy()), drawTools, 
		       "draw curved arrow" );
  drawCArrow->setPopup(BuildCurveArrowMenu());
  drawCArrow->setPopupDelay(1);
  QWhatsThis::add(drawCArrow, cArrowToolText);

  bracketIcon = QPixmap( brackettool );
  myfactory->setPixmap("brackettoolicon", bracketIcon );
  QToolButton * drawBracket
    = new QToolButton( bracketIcon, TOOLTIP_BRACKET, QString::null,
		       this, SLOT(DrawSquareBracket()), drawTools, 
		       "draw bracket" );
  drawBracket->setPopup(BuildBracketMenu());
  drawBracket->setPopupDelay(150);
  QWhatsThis::add(drawBracket, bracketToolText);

  textIcon = QPixmap( texttool );
  myfactory->setPixmap("texttoolicon", textIcon );
  QToolButton * drawText
    = new QToolButton( textIcon, TOOLTIP_TEXT, QString::null,
		       r, SLOT(setMode_DrawText()), drawTools, 
		       "draw or edit text" );
  QWhatsThis::add(drawText, textToolText);

  ringIcon = QPixmap( ringtool );
  myfactory->setPixmap("ringtoolicon", ringIcon );
  QToolButton * drawRing
    = new QToolButton( ringIcon, TOOLTIP_RING, QString::null,
		       this, SLOT(MakeRingDialog()), drawTools, 
		       "draw ring" );
  drawRing->setPopup(BuildNewRingMenu());
  drawRing->setPopupDelay(150);
  QWhatsThis::add(drawRing, ringToolText);

  symbolIcon = QPixmap( symboltool );
  myfactory->setPixmap("symboltoolicon", symbolIcon );
  QToolButton * drawSymbol
    = new QToolButton( symbolIcon, TOOLTIP_SYMBOL, QString::null,
		       this, SLOT(Dummy()), drawTools, "draw symbol" );
  drawSymbol->setPopup(BuildSymbolMenu());
  drawSymbol->setPopupDelay(1);
  QWhatsThis::add(drawSymbol, symbolToolText);

  // second (left? right?) toolbar - predefined rings
  ringTools = new QToolBar( this, "canned structures" );
  ringTools->setLabel( tr( "Canned Structures" ) );
  moveToolBar(ringTools, QMainWindow::Left);

  XDC_ToolButton *b1;
  QBitmap mask1(24, 24);
  mask1.fill(Qt::color1);

  b1 = new XDC_ToolButton( ringTools, "cyclopropane.cml");
  QToolTip::add(b1, tr("Cyclopropane") );
  px1 = QPixmap(n3ring);
  px1.setMask(mask1);
  b1->setPixmap( px1 );
  connect( b1, SIGNAL(pressed()), b1, SLOT(trigger()));
  connect( b1, SIGNAL(IncludeFile(QString)), this, 
	   SLOT(FromRingToolbar(QString)) );
  b1 = new XDC_ToolButton( ringTools, "cyclobutane.cml");
  QToolTip::add(b1, tr("Cyclobutane") );
  px1 = QPixmap(n4ring);
  px1.setMask(mask1);
  b1->setPixmap( px1 );
  connect( b1, SIGNAL(pressed()), b1, SLOT(trigger()));
  connect( b1, SIGNAL(IncludeFile(QString)), this, 
	   SLOT(FromRingToolbar(QString)) );
  b1 = new XDC_ToolButton( ringTools, "cyclopentane.cml");
  QToolTip::add(b1, tr("Cyclopentane") );
  px1 = QPixmap(n5ring);
  px1.setMask(mask1);
  b1->setPixmap( px1 );
  connect( b1, SIGNAL(pressed()), b1, SLOT(trigger()));
  connect( b1, SIGNAL(IncludeFile(QString)), this, 
	   SLOT(FromRingToolbar(QString)) );
  b1 = new XDC_ToolButton( ringTools, "cyclopentadiene.cml");
  QToolTip::add(b1, tr("Cyclopentadiene") );
  px1 = QPixmap(n5ring_diene);
  px1.setMask(mask1);
  b1->setPixmap( px1 );
  connect( b1, SIGNAL(pressed()), b1, SLOT(trigger()));
  connect( b1, SIGNAL(IncludeFile(QString)), this, 
	   SLOT(FromRingToolbar(QString)) );
  b1 = new XDC_ToolButton( ringTools, "cyclohexane.cml");
  QToolTip::add(b1, tr("Cyclohexane") );
  px1 = QPixmap(n6ring);
  px1.setMask(mask1);
  b1->setPixmap( px1 );
  connect( b1, SIGNAL(pressed()), b1, SLOT(trigger()));
  connect( b1, SIGNAL(IncludeFile(QString)), this, 
	   SLOT(FromRingToolbar(QString)) );
  b1 = new XDC_ToolButton( ringTools, "6ring_chair.cml");
  QToolTip::add(b1, tr("Cyclohexane - chair conformation") );
  px1 = QPixmap(n6ring_chair);
  px1.setMask(mask1);
  b1->setPixmap( px1 );
  connect( b1, SIGNAL(pressed()), b1, SLOT(trigger()));
  connect( b1, SIGNAL(IncludeFile(QString)), this, 
	   SLOT(FromRingToolbar(QString)) );
  b1 = new XDC_ToolButton( ringTools, "6ring_boat.cml");
  QToolTip::add(b1, tr("Cyclohexane - boat conformation") );
  px1 = QPixmap(n6ring_boat);
  px1.setMask(mask1);
  b1->setPixmap( px1 );
  connect( b1, SIGNAL(pressed()), b1, SLOT(trigger()));
  connect( b1, SIGNAL(IncludeFile(QString)), this, 
	   SLOT(FromRingToolbar(QString)) );
  b1 = new XDC_ToolButton( ringTools, "benzene.cml");
  QToolTip::add(b1, tr("Benzene") );
  px1 = QPixmap(n6ring_benzene);
  px1.setMask(mask1);
  b1->setPixmap( px1 );
  connect( b1, SIGNAL(pressed()), b1, SLOT(trigger()));
  connect( b1, SIGNAL(IncludeFile(QString)), this, 
	   SLOT(FromRingToolbar(QString)) );

  // menus
  QPopupMenu * file = new QPopupMenu( this );
  menuBar()->insertItem( FILE_MENU, file );

  file->insertItem( FILE_NEW, this, SLOT(newDoc()), CTRL+Key_N );

  id = file->insertItem( openIcon, FILE_OPEN,
			 this, SLOT(load()), CTRL+Key_O );
  file->setWhatsThis( id, fileOpenText );

  // add network option if "curl" is available
  FILE *test1;
  char *spam;
  QString cmd = "curl -s \"http://www.yahoo.com/\"";
  test1 = popen(cmd.ascii(), "r");
  spam = (char*)malloc(sizeof(char) * 65536);
  fread(spam, 1, 65535, test1);
  QString testq = spam;
  pclose(test1);
  free(spam);

  if (testq.length() > 1000) {
    id = file->insertItem( FILE_FIND_INTERNET, this,
			   SLOT(MakeNetDialog()), CTRL+Key_F );
  }

  id = file->insertItem( saveIcon, FILE_SAVE, 
			 this, SLOT(save()), CTRL+Key_S );
  file->setWhatsThis( id, fileSaveText );
  id = file->insertItem( FILE_SAVE_AS, this, SLOT(saveAs()) );
  file->setWhatsThis( id, fileSaveText );
  id = file->insertItem( FILE_SAVE_PICTURE, this, SLOT(savePicture()) );
  file->insertSeparator();
  id = file->insertItem( FILE_PAGE_SETUP, this , SLOT(PageSetup()) );
  id = file->insertItem( printIcon, FILE_PRINT,
			 this, SLOT(print()), CTRL+Key_P );
  file->setWhatsThis( id, filePrintText );
  file->insertSeparator();
  file->insertItem( FILE_CLOSE, this, SLOT(close()), CTRL+Key_W );
  file->insertItem( FILE_QUIT, qApp, SLOT( closeAllWindows() ), CTRL+Key_Q );

  QPopupMenu * rotateSub = new QPopupMenu(this);
  rotateSub->insertItem( EDIT_ROTATE_90CW, this, SLOT(Rotate90()) );
  rotateSub->insertItem( EDIT_ROTATE_180, this, SLOT(Rotate180()) );
  rotateSub->insertItem( EDIT_ROTATE_90CCW, this, SLOT(Rotate270()) );

  QPopupMenu * flipSub = new QPopupMenu(this);
  id = flipSub->insertItem( EDIT_FLIP_H, this, SLOT(FlipH()) );
  id = flipSub->insertItem( EDIT_FLIP_V, this, SLOT(FlipV()) );

  QPopupMenu * edit = new QPopupMenu(this);
  menuBar()->insertItem(EDIT_MENU, edit);

  edit->insertItem(EDIT_UNDO, this, SLOT(Undo()), CTRL+Key_Z);
  edit->insertSeparator();
  edit->insertItem(EDIT_CUT, this, SLOT(Cut()), CTRL+Key_X);
  edit->insertItem(EDIT_COPY, this, SLOT(Copy()), CTRL+Key_C);
  edit->insertItem(EDIT_PASTE, this, SLOT(Paste()), CTRL+Key_V);
  edit->insertItem(EDIT_CLEAR, this, SLOT(Clear()), Key_Delete);
  edit->insertSeparator();
  edit->insertItem(EDIT_SELECT_ALL, this, SLOT(SelectAll()), CTRL+Key_A);
  edit->insertItem(EDIT_ROTATE, rotateSub );
  edit->insertItem(EDIT_FLIP, flipSub );

  format = new QPopupMenu(this);
  format->setCheckable(true);
  menuBar()->insertItem(FORMAT_MENU, format);

  fixlen = format->insertItem(FORMAT_FIXED,this,SLOT(setFixed()) );
  format->setItemChecked(fixlen, true);	
  format->insertItem(FORMAT_SETFIXED, this, SLOT(ShowFixedDialog()));
  format->insertSeparator();
  format->insertItem(FORMAT_BACKGROUND, this, SLOT(BackgroundColor()) );

  QPopupMenu * tools = new QPopupMenu(this);
  menuBar()->insertItem(TOOLS_MENU, tools);
  tools->insertItem( tr("Clean up molecule"), this, SLOT(CleanUpMolecule()) );
  tools->insertItem(TOOLS_AUTOLAYOUT, this, SLOT(AutoLayout()), CTRL+Key_L );
  tools->insertSeparator();
  tools->insertItem(TOOLS_CALC_EF, this, SLOT(CalcEF()) );
  tools->insertItem(TOOLS_CALC_MW, this, SLOT(CalcMW()) );
  tools->insertItem(TOOLS_EA, this, SLOT(CalcEA()) );
  //tools->insertItem("IUPAC name", this, SLOT(CalcName()) );
  //tools->insertItem(tr("Get SMILES string"), this, SLOT(ToSMILES()) );
  //tools->insertItem("Predict 1H NMR", this, SLOT(Calc1HNMR()) );
  tools->insertItem(TOOLS_13CNMR, this, SLOT(Calc13CNMR()) );
  tools->insertItem(TOOLS_IR, this, SLOT(CalcIR()) );
  tools->insertSeparator();
  tools->insertItem(tr("Input SMILES"), this, SLOT(FromSMILES()) );

  QPopupMenu * help = new QPopupMenu( this );
  menuBar()->insertSeparator();
  menuBar()->insertItem( HELP_MENU, help );

  help->insertItem( HELP_MANUAL, this, SLOT(Manual()), Key_F1 );
  help->insertItem( HELP_ABOUT, this, SLOT(about()) );
  help->insertItem( HELP_REF, this, SLOT(Refs()) );
  help->insertSeparator();
  help->insertItem( "What's &This", this, SLOT(whatsThis()), SHIFT+Key_F1 );

  // create data system
  c = new ChemData(clip);

  // connect (non-Qt) data center and render widget
  r->setChemData(c);
  c->setRender2D(r);
  // connect renderer to application window
  connect(r, SIGNAL(TextOn(QFont)), this, SLOT(ShowTextButtons(QFont)) );
  connect(r, SIGNAL(TextOff()), this, SLOT(HideTextButtons()) );
  connect( r, SIGNAL(SignalSetStatusBar(QString)),
	   this, SLOT(SetStatusBar(QString)) );
  connect( c, SIGNAL(SignalSetStatusBar(QString)),
	   this, SLOT(SetStatusBar(QString)) );

  setCentralWidget( sv );
  statusBar()->message( "Ready", 2000 );
  resize( 640, 560 );
}

QPopupMenu *ApplicationWindow::BuildRingMenu() {
  QPopupMenu * ringSub = new QPopupMenu(this);
  QDir d(RingDir, "*.png");
  ringlist = d.entryList();
  for (int cc = 0; cc < ringlist.count(); cc++) {
    ringSub->insertItem( QPixmap( QString(RingDir + ringlist[cc]) ),
			 this, SLOT(FromRingMenu(int)), 0, cc );
  }
  return ringSub;
}

QPopupMenu *ApplicationWindow::BuildSymbolMenu() {
  QPopupMenu * symbolSub = new QPopupMenu(this);
  QPixmap p1;
  p1 = QPixmap(sym_plus_xpm);
  symbollist.append(QString("sym_plus"));
  symbolSub->insertItem( p1, this, SLOT(FromSymbolMenu(int)), 0, 0);
  p1 = QPixmap(sym_minus_xpm);
  symbollist.append(QString("sym_minus"));
  symbolSub->insertItem( p1, this, SLOT(FromSymbolMenu(int)), 0, 1);
  p1 = QPixmap(sym_delta_plus_xpm);
  symbollist.append(QString("sym_delta_plus"));
  symbolSub->insertItem( p1, this, SLOT(FromSymbolMenu(int)), 0, 2);
  p1 = QPixmap(sym_delta_minus_xpm);
  symbollist.append(QString("sym_delta_minus"));
  symbolSub->insertItem( p1, this, SLOT(FromSymbolMenu(int)), 0, 3);
  p1 = QPixmap(sym_1e_xpm);
  symbollist.append(QString("sym_1e"));
  symbolSub->insertItem( p1, this, SLOT(FromSymbolMenu(int)), 0, 4);
  p1 = QPixmap(sym_2e_xpm);
  symbollist.append(QString("sym_2e"));
  symbolSub->insertItem( p1, this, SLOT(FromSymbolMenu(int)), 0, 5);
  return symbolSub;
}

QPopupMenu *ApplicationWindow::BuildArrowMenu() {
  QPopupMenu * arrowSub = new QPopupMenu(this);
  QPixmap p1;
  p1 = QPixmap(arrow_regular_xpm);
  arrowlist.append(QString("REGULAR"));
  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 0);
  p1 = QPixmap(arrow_dash_xpm);
  arrowlist.append(QString("DASH"));
  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 1);
  p1 = QPixmap(arrow_bi1_xpm);
  arrowlist.append(QString("BI1"));
  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 2);
  p1 = QPixmap(arrow_bi2_xpm);
  arrowlist.append(QString("BI2"));
  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 3);
  p1 = QPixmap(arrow_retro_xpm);
  arrowlist.append(QString("RETRO"));
  arrowSub->insertItem( p1, this, SLOT(FromArrowMenu(int)), 0, 4);
  return arrowSub;
}

QPopupMenu *ApplicationWindow::BuildCurveArrowMenu() {
  QPopupMenu * arrowSub = new QPopupMenu(this);
  QPixmap p1;
  p1 = QPixmap(cw90_xpm);
  curvearrowlist.append(QString("CW90"));
  arrowSub->insertItem( p1, this, SLOT(FromCurveArrowMenu(int)), 0, 0);
  p1 = QPixmap(ccw90_xpm);
  curvearrowlist.append(QString("CCW90"));
  arrowSub->insertItem( p1, this, SLOT(FromCurveArrowMenu(int)), 0, 1);
  p1 = QPixmap(cw180_xpm);
  curvearrowlist.append(QString("CW180"));
  arrowSub->insertItem( p1, this, SLOT(FromCurveArrowMenu(int)), 0, 2);
  p1 = QPixmap(ccw180_xpm);
  curvearrowlist.append(QString("CCW180"));
  arrowSub->insertItem( p1, this, SLOT(FromCurveArrowMenu(int)), 0, 3);
  p1 = QPixmap(cw270_xpm);
  curvearrowlist.append(QString("CW270"));
  arrowSub->insertItem( p1, this, SLOT(FromCurveArrowMenu(int)), 0, 4);
  p1 = QPixmap(ccw270_xpm);
  curvearrowlist.append(QString("CCW270"));
  arrowSub->insertItem( p1, this, SLOT(FromCurveArrowMenu(int)), 0, 5);
  return arrowSub;
}

QPopupMenu *ApplicationWindow::BuildBracketMenu() {
  QPopupMenu * bracketSub = new QPopupMenu(this);
  QPixmap p1;
  p1 = QPixmap(squarebracket_xpm);
  bracketlist.append(QString("SQUARE"));
  bracketSub->insertItem( p1, this, SLOT(FromBracketMenu(int)), 0, 0);
  p1 = QPixmap(curvebracket_xpm);
  bracketlist.append(QString("CURVE"));
  bracketSub->insertItem( p1, this, SLOT(FromBracketMenu(int)), 0, 1);
  p1 = QPixmap(bracebracket_xpm);
  bracketlist.append(QString("BRACE"));
  bracketSub->insertItem( p1, this, SLOT(FromBracketMenu(int)), 0, 2);
  return bracketSub;
}

void ApplicationWindow::FromRingMenu(int x) {
  //QString dname(RINGHOME);
  //if (dname.left(1) != QString("/"))
  //  dname.append(QString("/"));
  c->StartUndo(0,0);
  c->DeselectAll();
  c->SetTopLeft(sv->viewportToContents(QPoint(0,0)));
  QString fname( ringlist[x] );
  fname.replace(fname.length() - 3, 3, QString("cml"));
  c->load( RingDir + fname );
  r->Inserted();
}

void ApplicationWindow::FromRingToolbar(QString fi) {
  c->StartUndo(0,0);
  c->DeselectAll();
  c->load( RingDir + fi );
  r->Inserted();
}

void ApplicationWindow::FromSymbolMenu(int x) {
  r->setMode_DrawSymbol( symbollist[x] );
}

void ApplicationWindow::FromArrowMenu(int x) {
  r->setMode_DrawArrow( arrowlist[x] );
}

void ApplicationWindow::FromCurveArrowMenu(int x) {
  r->setMode_DrawCurveArrow( curvearrowlist[x] );
}

void ApplicationWindow::FromBracketMenu(int x) {
  r->setMode_DrawBracket( bracketlist[x] );
}

void ApplicationWindow::setFixed()
{
  if (format->isItemChecked(fixlen)) {
    format->setItemChecked(fixlen, false);
    r->setFixed(false);
  } else {
    format->setItemChecked(fixlen, true);	
    r->setFixed(true);
  }
}

ApplicationWindow::~ApplicationWindow()
{
    delete printer;
}

void ApplicationWindow::SetFont(int m) {
  // read new font setting from combo boxes and pass to r  
  QFont f(fontList->currentText(), fontSizeList->currentText().toInt());
  cout << f.family() << f.pointSize() << endl;
  r->SetFont(f);
}

void ApplicationWindow::SetColor(int m) {
  if (m == 0) r->SetColor(QColor(0,0,0));
  if (m == 1) r->SetColor(QColor(127,0,0));
  if (m == 2) r->SetColor(QColor(0,127,0));
  if (m == 3) r->SetColor(QColor(0,0,127));
}

void ApplicationWindow::SetThick(int t) {
  r->SetThick(t + 1);
}

void ApplicationWindow::ShowTextButtons(QFont infont) {
  Bold->show();
  Italic->show();
  Underline->show();
  superScript->show();
  subScript->show();
  // set font combo boxes to object font (passed as infont)
  if (infont.family().lower() == QString("courier"))
    fontList->setCurrentItem(0);
  if (infont.family().lower() == QString("helvetica"))
    fontList->setCurrentItem(1);
  if (infont.family().lower() == QString("times"))
    fontList->setCurrentItem(2);
  if (infont.pointSize() == 8) fontSizeList->setCurrentItem(0);
  if (infont.pointSize() == 10) fontSizeList->setCurrentItem(1);
  if (infont.pointSize() == 12) fontSizeList->setCurrentItem(2);
  if (infont.pointSize() == 14) fontSizeList->setCurrentItem(3);
  if (infont.pointSize() == 18) fontSizeList->setCurrentItem(4);
  if (infont.pointSize() == 24) fontSizeList->setCurrentItem(5);
  if (infont.pointSize() == 32) fontSizeList->setCurrentItem(6);
}

void ApplicationWindow::HideTextButtons() {
  Bold->hide();
  Italic->hide();
  Underline->hide();
  superScript->hide();
  subScript->hide();
  // set font combo boxes to current font
  QFont infont = r->GetFont();
  if (infont.family().lower() == QString("courier"))
    fontList->setCurrentItem(0);
  if (infont.family().lower() == QString("helvetica"))
    fontList->setCurrentItem(1);
  if (infont.family().lower() == QString("times"))
    fontList->setCurrentItem(2);
  if (infont.pointSize() == 8) fontSizeList->setCurrentItem(0);
  if (infont.pointSize() == 10) fontSizeList->setCurrentItem(1);
  if (infont.pointSize() == 12) fontSizeList->setCurrentItem(2);
  if (infont.pointSize() == 14) fontSizeList->setCurrentItem(3);
  if (infont.pointSize() == 18) fontSizeList->setCurrentItem(4);
  if (infont.pointSize() == 24) fontSizeList->setCurrentItem(5);
  if (infont.pointSize() == 32) fontSizeList->setCurrentItem(6);
}

void ApplicationWindow::newDoc()
{
  ApplicationWindow *ed = new ApplicationWindow(clip);
  ed->show();
}

void ApplicationWindow::load()
{
  QFileDialog fd(QString::null, QString::null, 0, 0, TRUE);
  fd.setCaption("Load file");
  fd.setMode(QFileDialog::ExistingFile);
  QStringList filters;
  filters.append("All files (*)");
  filters.append("XDrawChem native (*.xdc)");
  filters.append("MDL Molfile (*.mol)");
  filters.append("CML - Chemical Markup Language (*.cml)");
  fd.setFilters(filters);  
  if ( fd.exec() == QDialog::Accepted )
    load( fd.selectedFile() );
}


void ApplicationWindow::load( QString fileName )
{
  QFile f(fileName);
  if ( !f.open(IO_ReadOnly) ) {
    QMessageBox::warning(0, "Couldn't open file", "Could not open the file: " + fileName);
    statusBar()->message(QString("Unable to load ") + fileName);
    return;
  }
  f.close();
  SelectAll();
  Clear();

  if ( !c->load(fileName) ) {
    statusBar()->message(QString("Unable to load ") + fileName);
    return;
  }
  
  setCaption( fileName );
  QString s("Loaded document ");
  statusBar()->message( s + fileName );
  filename = fileName;
  c->DeselectAll();
  r->repaint();
}


void ApplicationWindow::save()
{
  if ( filename.isEmpty() ) {
    saveAs();
    return;
  }
  
  if ( !c->save(filename) ) {
    statusBar()->message( QString("Could not write to %1").arg(filename) );
    return;
  }
  
  setCaption( filename );
  
  statusBar()->message( QString( "File %1 saved" ).arg( filename ) );
}


void ApplicationWindow::saveAs()
{
 QFileDialog fd(QString::null, QString::null, 0, 0, TRUE);
  fd.setCaption("Save file as...");
  fd.setMode(QFileDialog::AnyFile);
  QStringList filters;
  filters.append("XDrawChem native (*.xdc)");
  filters.append("MDL Molfile (*.mol)");
  filters.append("CML - Chemical Markup Language (*.cml)");
  //filters.append("Encapsulated PostScript (*.eps)");
  fd.setFilters(filters);  
  if ( fd.exec() == QDialog::Accepted ) {
    filename = fd.selectedFile();
    QString tmpx = filename.right(4).lower();
    if ( (tmpx != QString(".xdc")) &&
	 (tmpx != QString(".mol")) &&
	 (tmpx != QString(".cml")) ) {
      if (fd.selectedFilter().left(3) == QString("XDr"))
	filename.append(".xdc");
      if (fd.selectedFilter().left(3) == QString("MDL"))
	filename.append(".mol");
      if (fd.selectedFilter().left(3) == QString("CML"))
	filename.append(".cml");
    }
    save();
  }
}

void ApplicationWindow::savePicture()
{
  int pm = 0;
  bool was_saved;
  QString picfile;

  QFileDialog fd(QString::null, QString::null, 0, 0, TRUE);
  fd.setCaption("Save as picture...");
  fd.setMode(QFileDialog::AnyFile);
  QStringList filters;
  filters.append("Portable Network Graphic (*.png)");
  filters.append("Windows Bitmap (*.bmp)");
  filters.append("Encapsulated PostScript (*.eps)");
  fd.setFilters(filters);  
  if ( fd.exec() == QDialog::Accepted ) {
    picfile = fd.selectedFile();
    QString tmpx = picfile.right(4).lower();
    if (tmpx == QString(".png")) pm = 1;
    if (tmpx == QString(".bmp")) pm = 2;
    if ( (tmpx != QString(".png")) &&
	 (tmpx != QString(".bmp")) &&
	 (tmpx != QString(".eps")) ) {
      if (fd.selectedFilter().left(3) == QString("Por")) {
	picfile.append(".png"); pm = 1;
      }
      if (fd.selectedFilter().left(3) == QString("Win")) {
	picfile.append(".bmp"); pm = 2;
      }
      if (fd.selectedFilter().left(3) == QString("Enc")) {
	picfile.append(".eps");
      }
    }
    if (pm != 0) {
      QPixmap tosave = r->MakePixmap();
      if (pm == 1) // PNG
	was_saved = tosave.save(picfile, "PNG");
      if (pm == 2) // BMP
	was_saved = tosave.save(picfile, "BMP");
      if (was_saved)
	statusBar()->message( QString( "File %1 saved" ).arg( picfile ) );
      else
	statusBar()->message("Unable to save picture!");
    } else { // save as EPS
      was_saved = r->SaveEPS(picfile);
      if (was_saved)
	statusBar()->message( QString( "File %1 saved" ).arg( picfile ) );
      else
	statusBar()->message("Unable to save picture!");
    }
  }
}

void ApplicationWindow::print()
{
  r->Print();
}

void ApplicationWindow::closeEvent( QCloseEvent* ce )
{
    if ( !c->edited() ) {
	ce->accept();
	return;
    }

    switch( QMessageBox::information( this, "XDrawChem question",
				      "The document has been changed since "
				      "the last save.",
				      "Save Now", "Cancel", "Leave Anyway",
				      0, 1 ) ) {
    case 0:
	save();
	ce->accept();
	break;
    case 1:
    default: // just for sanity
	ce->ignore();
	break;
    case 2:
	ce->accept();
	break;
    }
}


void ApplicationWindow::about()
{
    QMessageBox::about( this, XDC_VERSION,
			XDC_VERSION"\nBryan Herger\nherger@chemistry.gatech.edu\n\nPortions copyright (C) 1997-2000 Dr. Christoph Steinbeck\nSee file COPYRIGHT.txt for more details");
}

const char * reflist = 
"Krause S, Willighagen E, Steinbeck C, "
"\"JChemPaint - Using the Collaborative Forces of the Internet\nto Develop "
"a Free Editor for 2D Chemical Structures\", Molecules 5:93-98\n\n"
"Bremser W, \"HOSE - A Novel Substructure Code\", Anal. Chim. Acta 103:"
"355-365\n\n"
"Bremser W, \"Expectation Ranges of 13C NMR Chemical Shifts\", "
"Mag. Res. Chem. 23(4):271-275\n\n"
"Weininger D, \"SMILES, a Chemical Language and Information System.  1. Introduction to Methodology and Encoding Rules\", J. Chem. Inf. Comput. Sci. 28:31-36\n\n"
"Figueras J, \"Ring Perception Using Breadth-First Search\", J. Chem. Inf. Comput Sci. 36:986-991\n\n"
"Ugi I et al., Journal of Chemical research (M), 1991, 2601-2689\n\n"
"Steinbeck C, JMDraw (software), http://jmdraw.sourceforge.net/\n\n"
"Molecules in the XDrawChem database are from the NCI Open Database, October 1999 release:  http://cactvs.cit.nih.gov/";

void ApplicationWindow::Refs()
{
  QMessageBox::about( this, "XDrawChem 0.9 References", reflist);
}

void ApplicationWindow::SetStatusBar(QString s) {
  statusBar()->message(s);
}

void ApplicationWindow::MakeRingDialog() {
  RingDialog t(this, "ring dialog");
  if ( !t.exec() ) return;

  c->StartUndo(0,0);
  c->DeselectAll();
  c->SetTopLeft(sv->viewportToContents(QPoint(0,0)));
  c->load( RingDir + t.getFile() );
  r->Inserted();
}

void ApplicationWindow::ShowFixedDialog(void) {
  double na, nl;
  FixedDialog i(this, QString("fixed_dialog"));
  i.setAngle( r->getFixedAngle() );
  i.setLength( r->getFixedLength() );
  if ( !i.exec() ) return;
  na = i.getAngle();
  nl = i.getLength();
  if (na < 0.0) return;
  if (nl < 0.0) return;
  r->setFixedAngleAndLength(na, nl);
}

void ApplicationWindow::MakeNetDialog() {
  NetDialog n(this, "net dialog");
  if ( !n.exec() ) return;
  cout << "Server:" << n.getServer() << endl;
  cout << "Key   :" << n.getKey() << endl;
  cout << "Value :" << n.getValue() << endl;
  NetAccess na;
  setCursor(waitCursor);
  r->setWaitCursor();
  QStringList choices = na.getChoices(n.getServer(), n.getKey(), n.getValue(),
				      n.getExact() );
  setCursor(arrowCursor);
  r->setArrowCursor();
  if (choices.count() == 0) {
    QMessageBox::warning( this, "Database query failed", "No molecules in the database match the query." );
    return;
  }
  if (choices.count() == 1) {
    int i1 = choices[0].find("|");
    QString subfile = choices[0].left(i1) + ".mol";
    QString wf = na.getFile(n.getServer(), subfile);
    c->DeselectAll();
    c->SetTopLeft(sv->viewportToContents(QPoint(0,0)));
    c->ProcessMDL(wf);
    r->Inserted();
    return;
  }
  NetChooseDialog nc(this, "net choose dialog", choices);
  if ( !nc.exec() ) return;
  cout << nc.getFile() << endl;
  QString wf = na.getFile(n.getServer(), nc.getFile());
  if (wf.contains("file not found") > 0) {
    QMessageBox::warning( this, "Database query failed", "An error occured.  The server cannot find the selected molecule." );
    return;
  }
  if (nc.getFile().contains(".mol")) {
    c->DeselectAll();
    c->SetTopLeft(sv->viewportToContents(QPoint(0,0)));
    c->ProcessMDL(wf);
    r->Inserted();
    return;
  }
}

void ApplicationWindow::PageSetup() {
  PageSetupDialog p(this, "page dialog");
  p.setPageSize(r->getPageSize());
  p.setOrientation(r->getPageOrientation());
  if ( !p.exec() ) return;
  r->setPageSizeAndOrientation(p.getPageSize(), p.getOrientation());
}

void ApplicationWindow::NewColor() {
  QColor nc1 = QColorDialog::getColor(r->GetColor());
  if (nc1.isValid()) {
    r->SetColor(nc1);
    QPixmap *tmp_pm = new QPixmap(32, 20);
    tmp_pm->fill(QColor(210,210,210));
    QPainter pixpaint(tmp_pm);
    pixpaint.fillRect(2, 3, 14, 14, nc1);
    pixpaint.setPen(QColor(0,0,0));
    pixpaint.drawLine( 24, 2, 24, 16 );
    pixpaint.drawLine( 24, 16, 20, 12 );
    pixpaint.drawLine( 24, 16, 28, 12 );
    pb1->setPixmap(*tmp_pm);    
  }
}

void ApplicationWindow::BackgroundColor() {
  QColor nc1 = QColorDialog::getColor(r->getBGColor());
  if (nc1.isValid()) {
    r->setBGColor(nc1);
    r->repaint();
  }
}

// pass thru to Render2D
void ApplicationWindow::Cut() { r->Cut(); }
void ApplicationWindow::Copy() { r->Copy(); }
void ApplicationWindow::Paste() { r->Paste(); }
void ApplicationWindow::Undo() { r->Undo(); }
void ApplicationWindow::FlipH() { r->Flip(FLIP_H); }
void ApplicationWindow::FlipV() { r->Flip(FLIP_V); }
void ApplicationWindow::Rotate90() { r->Rotate90(); }
void ApplicationWindow::Rotate180() { r->Rotate180(); }
void ApplicationWindow::Rotate270() { r->Rotate270(); }
void ApplicationWindow::Clear() { r->EraseSelected(); }
void ApplicationWindow::CalcEA() { r->Tool(MODE_TOOL_CALCEA); }
void ApplicationWindow::CalcEF() { r->Tool(MODE_TOOL_CALCEF); }
void ApplicationWindow::CalcMW() { r->Tool(MODE_TOOL_CALCMW); }
void ApplicationWindow::Calc13CNMR() { r->Tool(MODE_TOOL_13CNMR); }
void ApplicationWindow::Calc1HNMR() { r->Tool(MODE_TOOL_1HNMR); }
void ApplicationWindow::CalcIR() { r->Tool(MODE_TOOL_IR); }
void ApplicationWindow::CalcName() { r->Tool(MODE_TOOL_NAME); }
void ApplicationWindow::ToSMILES() { r->Tool(MODE_TOOL_TOSMILES); }
void ApplicationWindow::CleanUpMolecule() { r->Tool(MODE_TOOL_CLEANUPMOL); }
void ApplicationWindow::SelectAll() { r->SelectAll(); }
void ApplicationWindow::AutoLayout() { r->AutoLayout(); }
void ApplicationWindow::DrawRegularArrow() {
  r->setMode_DrawArrow( "REGULAR" );
}
void ApplicationWindow::DrawSquareBracket() {
  r->setMode_DrawBracket( "SQUARE" );
}
